# This is the test for the simple associative memory that we use in the paper.
#
#
import sys

import numpy as np
import pyNN.nest as sim
import cPickle as pickle
from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from stableBumpClass import StableBump

#
COFFEE = 0
COKE = 1
COLD = 2
WARM = 3
HOT = 4

#
def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

def connectInhib(inhibPopulation, aTemperaturePopulation):
    toInhibWt = 0.04
    fromInhibWt = 0.016
    for temp in range(0,100):
        for inhibOffset in range(0,10):
            fsa.oneNeuronStimulatesOneNeuron(aTemperaturePopulation,temp,
                                         inhibPopulation,inhibOffset,toInhibWt)
            fsa.oneNeuronInhibitsOneNeuron(inhibPopulation,inhibOffset,
                    aTemperaturePopulation,temp,fromInhibWt)

def buildHotCoffeeTopology(semanticPopulation, temperaturePopulation,
                           coffeePopulation, cokePopulation,inhibPopulation):
    connectInhib(inhibPopulation,cokePopulation)
    connectInhib(inhibPopulation,coffeePopulation)
    fsa.makeCA(semanticPopulation,COFFEE) # coffee 0
    fsa.makeCA(semanticPopulation,COKE) # coke 1
    fsa.makeCA(semanticPopulation,COLD) # cold 2
    fsa.makeCA(semanticPopulation,WARM) # warm 3
    fsa.makeCA(semanticPopulation,HOT) # hot 4

    toTempWt = 0.0015 
    tempToSemWt = 0.025
    semToTempWt = 0.001 

    tempToDrinkWt = 0.01
    #input temp stimulates coffee and coke temp
    for temp in range(0,100):
	fsa.oneNeuronStimulatesOneNeuron(temperaturePopulation,temp,
                                         coffeePopulation,temp,tempToDrinkWt)
	fsa.oneNeuronStimulatesOneNeuron(temperaturePopulation,temp,
                                         cokePopulation,temp,tempToDrinkWt)
        #coffee-> coffeepop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COFFEE,
                                         coffeePopulation,temp,semToTempWt)
        #coke -> cokepop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COKE,
                                         cokePopulation,temp,semToTempWt)

    #coffee <-> temperatures
    for temp in range(0,25):
        #coffeePop -> coffee
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COFFEE,tempToSemWt)
        #coffeePop -> cold
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COLD,tempToSemWt)
        #cold-> cofeePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COLD,
                                     coffeePopulation,temp,toTempWt)

    for temp in range(25,70): 
        #coffeePop -> coffee
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COFFEE,tempToSemWt)
        #coffeePop -> warm
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,WARM,tempToSemWt)
        #warm-> cofeePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,WARM,
                                     coffeePopulation,temp,toTempWt)


    for temp in range(70,100):
        #coffeePop -> coffee
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,COFFEE,tempToSemWt)
        #coffeePop -> hot
	fsa.oneNeuronStimulatesState(coffeePopulation,temp,
                                     semanticPopulation,HOT,tempToSemWt)
        #hot-> cofeePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,HOT,
                                     coffeePopulation,temp,toTempWt)

    #coke <-> temperatures
    for temp in range(0,10):
        #cokePop -> coke
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COKE,tempToSemWt)
        #cokePop -> cold  
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COLD,tempToSemWt)
        #cold-> cokePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,COLD,
                                     cokePopulation,temp,toTempWt)

    #coke <-> temperatures

    for temp in range(10,24):
        #cokePop -> coke
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COKE,tempToSemWt)
        #cokePop -> warm
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,WARM,tempToSemWt)
        #warm-> cokePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,WARM,
                                     cokePopulation,temp,toTempWt)

    for temp in range(25,100):
        #cokePop -> coke
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,COKE,tempToSemWt)
        #cokePop -> hot
	fsa.oneNeuronStimulatesState(cokePopulation,temp,
                                     semanticPopulation,HOT,tempToSemWt)
        #hot-> cokePop
	fsa.stateStimulatesOneNeuron(semanticPopulation,HOT,
                                     cokePopulation,temp,toTempWt)

#--main
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    testType  = int(args[1])
else:
    testType = 12
print testType


simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)
stableBump = StableBump(neal,fsa)

init()

neurons = 100
cellClass=sim.IF_cond_exp
cellParams=fsa.CELL_PARAMS
tempPop = stableBump.stableBump(neurons,cellClass,cellParams,2,4,0.08,0.08)
stableBump.setupRecording(tempPop)

#coffeePop = sim.Population(100,cellClass,cellParams)
#cokePop = sim.Population(100,cellClass,cellParams)
coffeePop = stableBump.stableBump(neurons,cellClass,cellParams,2,4,0.08,0.08)
cokePop = stableBump.stableBump(neurons,cellClass,cellParams,2,4,0.08,0.08)
coffeePop.record('spikes')
cokePop.record('spikes')

pop = sim.Population(50,cellClass,cellParams)
pop.record('spikes')
inhibPop = sim.Population(10,cellClass,cellParams)
inhibPop.record('spikes')
buildHotCoffeeTopology(pop,tempPop,coffeePop,cokePop,inhibPop)

#TEST 
source0 = stableBump.createTwoInputs()
spikeSource = stableBump.createTwoInputs()
if testType == 1: # (coffee)->  just coffeee
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
elif testType == 2: # (coke)->  just coke
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)#coke
elif testType == 3: # (cold) -> just cold
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COLD)#cold
elif testType == 4: # (warm) -> just warm
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,WARM)
elif testType == 5: # hot
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,HOT)

elif testType == 6: # temperature turns on nothing
    for inputTemp in range (10,13):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)

elif testType == 7: # (coffee and cold) -> temp 0-24
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COLD)#cold
elif testType == 8: # temperature & cold -> coffee
    for inputTemp in range (14,17):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COLD)#cold
elif testType == 9: # coffee & temperature -> cold
    for inputTemp in range (10,13):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COFFEE)#coffee

elif testType == 10: # (coffee and warm) -> temp 25-69
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,WARM)#cold
elif testType == 11: # temperature & warm -> coffee
    for inputTemp in range (36,40):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,WARM)#warm
elif testType == 12: # temperature & coffee -> warm
    for inputTemp in range (36,40):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COFFEE)

elif testType == 13: # (coffee and hot) -> temp 80
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COFFEE)#coffee
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,HOT)
elif testType == 14: # temperature-90 & hot -> coffee
    for inputTemp in range (88,91):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,HOT)#hot
elif testType == 15: # temperature-90 & coffee -> hot
    for inputTemp in range (78,81):
        stableBump.turnOnCellFromSpikeSource(spikeSource[0],tempPop,inputTemp)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COFFEE)

elif testType == 16: # (coke and cold) -> temp 0-4
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,COLD)#cold
elif testType == 17: # (coke and temperate-3 ) -> cold
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    for inputTemp in range (2,5):
        stableBump.turnOnCellFromSpikeSource(spikeSource[1],tempPop,inputTemp)
elif testType == 18: # (coke and temperate-5 ) -> coke
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COLD)
    for inputTemp in range (1,4):
        stableBump.turnOnCellFromSpikeSource(spikeSource[1],tempPop,inputTemp)

elif testType == 19: # (coke and warm) -> temp 11
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,WARM)
elif testType == 20: # (coke and temperate-11 ) -> WARM
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    for inputTemp in range (15,18):
        stableBump.turnOnCellFromSpikeSource(spikeSource[1],tempPop,inputTemp)
elif testType == 21: # (warm and temperate-11 ) -> coke
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,WARM)
    for inputTemp in range (14,17):
        stableBump.turnOnCellFromSpikeSource(spikeSource[1],tempPop,inputTemp)

elif testType == 22: # (coke and hot) -> temp-50
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    fsa.turnOnStateFromSpikeSource(spikeSource[1],pop,HOT)
elif testType == 23: # (coke and temp-50) -> HOT
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,COKE)
    for inputTemp in range (48,51):
        stableBump.turnOnCellFromSpikeSource(spikeSource[1],tempPop,inputTemp)
elif testType == 24: # (hot and temp-50) -> COKE
    fsa.turnOnStateFromSpikeSource(spikeSource[0],pop,HOT)
    for inputTemp in range (48,51):
        stableBump.turnOnCellFromSpikeSource(spikeSource[1],tempPop,inputTemp)

else: print "No testType=", testType


neal.nealApplyProjections()
sim.run(300)
sim.end()

tempPopPkl = tempPop.write_data("outputs/popTemp.pkl")
coffeePopPkl = coffeePop.write_data("outputs/coffeeTemp.pkl")
cokePopPkl = cokePop.write_data("outputs/cokeTemp.pkl")
popPkl = pop.write_data("outputs/semPop.pkl")
inhibPopPkl = inhibPop.write_data("outputs/inhibPop.pkl")
